@@ -7,6 +7,7 @@
 import os
 import sys
 import time
+import traceback
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
@@ -18,6 +19,7 @@ from kuwo import Config
 # ~/.config/kuwo and ~/.cache/kuwo need to be created at first time
 Config.check_first()
 _ = Config._
+from kuwo.log import logger
 from kuwo import Widgets
 from kuwo.Artists import Artists
 from kuwo.Lrc import Lrc
@@ -31,6 +33,11 @@ from kuwo.Shortcut import Shortcut
 from kuwo.Themes import Themes
 from kuwo.TopCategories import TopCategories
 from kuwo.TopList import TopList
+try:
+    # Ubuntu Unity uses AppIndicator instead of Gtk.StatusIcon
+    from gi.repository import AppIndicator3 as AppIndicator
+except ImportError:
+    logger.debug(traceback.format_exc())
 
 if Gtk.MAJOR_VERSION <= 3 and Gtk.MINOR_VERSION < 10:
     GObject.threads_init()
@@ -211,42 +218,40 @@ class App:
                 self.conf.get('use-dark-theme', False)
 
     def init_status_icon(self):
-        # set status_icon as class property, to keep its life
-        # after function exited
-        self.status_icon = Gtk.StatusIcon()
-        self.status_icon.set_from_pixbuf(self.theme['app-logo'])
-        # left click
-        self.status_icon.connect('activate', self.on_status_icon_activate)
-        # right click
-        self.status_icon.connect('popup_menu', self.on_status_icon_popup_menu)
+        def on_status_icon_popup_menu(status_icon, event_button,
+                                      event_time):
+            menu.popup(None, None,
+                    lambda a,b: Gtk.StatusIcon.position_menu(menu, status_icon),
+                    None, event_button, event_time)
 
-    def on_status_icon_activate(self, status_icon):
-        if self.window.props.visible:
-            self.window.hide()
-        else:
-            self.window.present()
+        def on_status_icon_activate(tatus_icon):
+            if self.window.props.visible:
+                self.window.hide()
+            else:
+                self.window.present()
+
+        if not self.conf['use-status-icon']:
+            return
 
-    def on_status_icon_popup_menu(self, status_icon, event_button, 
-                                  event_time):
         menu = Gtk.Menu()
 
-        show_item = Gtk.MenuItem(label=_('Show App') )
-        show_item.connect('activate', self.on_status_icon_show_app_activate)
+        show_item = Gtk.MenuItem(_('Show App'))
+        show_item.connect('activate', lambda item: self.window.present())
         menu.append(show_item)
 
         sep_item = Gtk.SeparatorMenuItem()
         menu.append(sep_item)
 
-        previous_item = Gtk.MenuItem(label=_('Previous Song'))
-        previous_item.connect('activate', self.on_status_icon_prev_activate)
+        previous_item = Gtk.MenuItem(_('Previous Song'))
+        previous_item.connect('activate', lambda item: self.player.load_prev())
         menu.append(previous_item)
 
         play_item = Gtk.MenuItem()
         play_item.props.related_action = self.player.playback_action
         menu.append(play_item)
 
-        next_item = Gtk.MenuItem(label=_('Next Song'))
-        next_item.connect('activate', self.on_status_icon_next_activate)
+        next_item = Gtk.MenuItem(_('Next Song'))
+        next_item.connect('activate', lambda item: self.player.load_next())
         menu.append(next_item)
 
         sep_item = Gtk.SeparatorMenuItem()
@@ -263,39 +268,33 @@ class App:
         sep_item = Gtk.SeparatorMenuItem()
         menu.append(sep_item)
         
-        quit_item = Gtk.MenuItem(label=_('Quit'))
-        quit_item.connect('activate', self.on_status_icon_quit_activate)
+        quit_item = Gtk.MenuItem(_('Quit'))
+        quit_item.connect('activate', lambda item: self.quit())
         menu.append(quit_item)
 
         menu.show_all()
-        menu.popup(None, None,
-                lambda a,b: Gtk.StatusIcon.position_menu(menu, status_icon),
-                None, event_button, event_time)
+        self.status_menu = menu
 
-    def on_status_icon_show_app_activate(self, menuitem):
-        self.window.present()
-
-    def on_status_icon_prev_activate(self, menuitem):
-        self.player.load_prev()
-
-    def on_status_icon_pause_activate(self, menuitem):
-        self.player.play_pause()
-
-    def on_status_icon_next_activate(self, menuitem):
-        self.player.load_next()
-
-    def on_status_icon_lock_osd_activate(self, menuitem):
-        self.osdlrc.toggle_lock()
-
-    def on_status_icon_quit_activate(self, menuitem):
-        self.quit()
+        if 'AppIndicator' in globals():
+            self.status_icon = AppIndicator.Indicator.new(Config.NAME,
+                    Config.NAME,
+                    AppIndicator.IndicatorCategory.APPLICATION_STATUS)
+            self.status_icon.set_menu(menu)
+            self.status_icon.set_status(AppIndicator.IndicatorStatus.ACTIVE)
+        else: 
+            self.status_icon = Gtk.StatusIcon()
+            self.status_icon.set_from_pixbuf(self.theme['app-logo'])
+            # left click
+            self.status_icon.connect('activate', on_status_icon_activate)
+            # right click
+            self.status_icon.connect('popup_menu', on_status_icon_popup_menu)
 
     def on_notebook_switch_page(self, notebook, page, page_num):
         if page not in self.tab_first_show:
             page.first()
             self.tab_first_show.append(page)
 
-        if page_num == 0:
+        if page_num == 0 and self.conf['auto-hide-tabs']:
             # fullscreen
             self.player.hide()
             self.notebook.set_show_tabs(False)
@@ -307,16 +306,22 @@ class App:
             self.notebook.set_show_tabs(True)
             self.fullscreen_sid = 0
 
+    def reset_notebook_tabs(self):
+        if not self.conf['auto-hide-tabs']:
+            self.player.show_all()
+            self.notebook.set_show_tabs(True)
+            self.fullscreen_sid = 0
+
     def on_window_motion_notified(self, window, event):
-        if self.notebook.get_current_page() != 0:
+        if (self.notebook.get_current_page() != 0 or
+                not self.conf['auto-hide-tabs']):
             return
         lower = 70
         upper = self.window.get_size()[1] - 40
         if lower < event.y < upper:
             return
-        if event.y < lower:
+        else:
             self.player.show_all()
-        elif event.y > upper:
             self.notebook.set_show_tabs(True)
         # delay 2 seconds and hide them
         self.fullscreen_timestamp = time.time()
@@ -325,8 +330,8 @@ class App:
                          self.fullscreen_timestamp)
 
     def hide_control_panel_and_label(self, timestamp):
-        if (timestamp == self.fullscreen_timestamp and 
-                self.fullscreen_sid > 0):
-            self.notebook.set_show_tabs(False)
+        if timestamp == self.fullscreen_timestamp and self.fullscreen_sid > 0:
             self.player.hide()
             GLib.timeout_add(100, self.player.playbin.expose)
+            #GLib.idle_add(self.player.playbin.expose)
+            self.notebook.set_show_tabs(False)
