@@ -10,10 +10,11 @@ from gi.repository import GdkPixbuf
 from gi.repository import Gtk
 
 from kuwo import Config
+_ = Config._
 from kuwo import Net
 from kuwo import Widgets
+from kuwo.log import logger
 
-_ = Config._
 
 class TopList(Gtk.Box):
     '''TopList tab in notebook.'''
@@ -45,11 +46,11 @@ class TopList(Gtk.Box):
         self.scrolled_nodes = Gtk.ScrolledWindow()
         self.pack_start(self.scrolled_nodes, True, True, 0)
         # logo, name, nid, info, tooltip
-        self.liststore_nodes = Gtk.ListStore(
-                GdkPixbuf.Pixbuf, str, int, str, str)
+        self.liststore_nodes = Gtk.ListStore(GdkPixbuf.Pixbuf, str, int,
+                                             str, str)
         iconview_nodes = Widgets.IconView(self.liststore_nodes, tooltip=4)
-        iconview_nodes.connect(
-                'item_activated', self.on_iconview_nodes_item_activated)
+        iconview_nodes.connect('item_activated',
+                               self.on_iconview_nodes_item_activated)
         self.scrolled_nodes.add(iconview_nodes)
 
         self.scrolled_songs = Gtk.ScrolledWindow()
@@ -60,27 +61,32 @@ class TopList(Gtk.Box):
         self.buttonbox.hide()
         self.scrolled_songs.hide()
 
+        def _on_get_nodes(info, error):
+            if error or not info or not info[0] or not info[1]:
+                logger.errror('_on_get_nodes(), info: %s, error: %s' %
+                              (info, error))
+                return
+            nodes, total_pages = info
+            urls = []
+            tree_iters = []
+            for node in nodes:
+                tree_iter = self.liststore_nodes.append([
+                    self.app.theme['anonymous'],
+                    Widgets.unescape(node['name']),
+                    int(node['sourceid']),
+                    Widgets.unescape(node['info']),
+                    Widgets.set_tooltip_with_song_tips(node['name'],
+                                                       node['tips']),
+                ])
+                urls.append(node['pic'])
+                tree_iters.append(tree_iter)
+            self.liststore_nodes.timestamp = time.time()
+            Net.async_call(Net.update_liststore_images, self.liststore_nodes, 0,
+                           tree_iters, urls)
+
         nid = 2
         page = 0
-        # TODO: use async call
-        nodes, total_pages = Net.get_nodes(nid, page)
-        if total_pages == 0:
-            return
-        urls = []
-        tree_iters = []
-        for node in nodes:
-            tree_iter = self.liststore_nodes.append([
-                self.app.theme['anonymous'],
-                Widgets.unescape(node['name']),
-                int(node['sourceid']),
-                Widgets.unescape(node['info']),
-                Widgets.set_tooltip_with_song_tips(
-                    node['name'], node['tips']),
-                ])
-            urls.append(node['pic'])
-            tree_iters.append(tree_iter)
-        self.liststore_nodes.timestamp = time.time()
-        Net.update_liststore_images(self.liststore_nodes, 0, tree_iters, urls)
+        Net.async_call(Net.get_nodes, nid, page, callback=_on_get_nodes)
 
     def on_button_home_clicked(self, btn):
         self.scrolled_nodes.show_all()
@@ -95,22 +101,26 @@ class TopList(Gtk.Box):
         self.show_toplist_songs(model[path][2])
 
     def show_toplist_songs(self, nid):
+        def _on_get_toplist_songs(songs, error):
+            if not songs or error:
+                logger.error('show_toplist_songs(), songs: %s, error: %s' %
+                             (songs, error))
+                return
+            self.liststore_songs.clear()
+            for song in songs:
+                self.liststore_songs.append([
+                    True,
+                    Widgets.unescape(song['name']), 
+                    Widgets.unescape(song['artist']),
+                    Widgets.unescape(song['album']),
+                    int(song['id']), 
+                    int(song['artistid']),
+                    int(song['albumid']),
+                    song['formats'],
+                ])
+
         self.scrolled_nodes.hide()
         self.scrolled_songs.show_all()
-
-        songs = Net.get_toplist_songs(nid)
-        if not songs:
-            print('Error, failed to get toplist songs')
-            return
-        self.liststore_songs.clear()
-        for song in songs:
-            self.liststore_songs.append([
-                True,
-                Widgets.unescape(song['name']), 
-                Widgets.unescape(song['artist']),
-                Widgets.unescape(song['album']),
-                int(song['id']), 
-                int(song['artistid']),
-                int(song['albumid']),
-                song['formats'],
-            ])
+        self.scrolled_songs.get_vscrollbar().set_value(0)
+        Net.async_call(Net.get_toplist_songs, nid,
+                       callback=_on_get_toplist_songs)
